@@ -1,3 +1,5 @@
+from collections import Sequence
+from distutils.version import LooseVersion
 import logging
 import warnings
 import sys
@@ -6,23 +8,58 @@ from os import path as op
 import inspect
 from functools import wraps
 
+import mayavi
+from mayavi import mlab
+from mayavi.filters.api import Threshold
 import numpy as np
 import nibabel as nib
 from scipy import sparse
 from scipy.spatial.distance import cdist
 import matplotlib as mpl
-from matplotlib import cm
+from matplotlib import cm as mpl_cm
+from . import cm as surfer_cm
 
 logger = logging.getLogger('surfer')
 
 
 # Py3k compat
 if sys.version[0] == '2':
-    string_types = basestring  # noqa
+    string_types = basestring  # noqa, analysis:ignore
 else:
     string_types = str
 
 
+if LooseVersion(mayavi.__version__) == LooseVersion('4.5.0'):
+    # Monkey-patch Mayavi 4.5:
+    # In Mayavi 4.5, filters seem to be missing a .point_data attribute that
+    # Threshold accesses on initialization.
+    _orig_meth = Threshold._get_data_range
+
+    def _patch_func():
+        return []
+
+    def _patch_meth(self):
+        return []
+
+    class _MayaviThresholdPatch(object):
+
+        def __enter__(self):
+            Threshold._get_data_range = _patch_meth
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            Threshold._get_data_range = _orig_meth
+
+    _mayavi_threshold_patch = _MayaviThresholdPatch()
+
+    def threshold_filter(*args, **kwargs):
+        with _mayavi_threshold_patch:
+            thresh = mlab.pipeline.threshold(*args, **kwargs)
+        thresh._get_data_range = _patch_func
+        return thresh
+else:
+    threshold_filter = mlab.pipeline.threshold
+
+
 class Surface(object):
     """Container for surface object
 
@@ -34,27 +71,18 @@ class Surface(object):
         Which hemisphere to load
     surf : string
         Name of the surface to load (eg. inflated, orig ...)
-    data_path : string
-        Path where to look for data
-    x: 1d array
-        x coordinates of vertices
-    y: 1d array
-        y coordinates of vertices
-    z: 1d array
-        z coordinates of vertices
-    coords : 2d array of shape [n_vertices, 3]
-        The vertices coordinates
-    faces : 2d array
-        The faces ie. the triangles
-    nn : 2d array
-        Normalized surface normals for vertices.
     subjects_dir : str | None
         If not None, this directory will be used as the subjects directory
         instead of the value set using the SUBJECTS_DIR environment variable.
+    offset : float | None
+        If float, align inside edge of each hemisphere to center + offset.
+        If None, do not change coordinates (default).
+    units : str
+        Can be 'm' or 'mm' (default).
     """
 
     def __init__(self, subject_id, hemi, surf, subjects_dir=None,
-                 offset=None):
+                 offset=None, units='mm'):
         """Surface
 
         Parameters
@@ -76,6 +104,10 @@ class Surface(object):
         self.hemi = hemi
         self.surf = surf
         self.offset = offset
+        self.coords = None
+        self.faces = None
+        self.nn = None
+        self.units = _check_units(units)
 
         subjects_dir = _get_subjects_dir(subjects_dir)
         self.data_path = op.join(subjects_dir, subject_id)
@@ -83,18 +115,24 @@ class Surface(object):
     def load_geometry(self):
         surf_path = op.join(self.data_path, "surf",
                             "%s.%s" % (self.hemi, self.surf))
-        self.coords, self.faces = nib.freesurfer.read_geometry(surf_path)
+        coords, faces = nib.freesurfer.read_geometry(surf_path)
+        if self.units == 'm':
+            coords /= 1000.
         if self.offset is not None:
             if self.hemi == 'lh':
-                self.coords[:, 0] -= (np.max(self.coords[:, 0]) + self.offset)
+                coords[:, 0] -= (np.max(coords[:, 0]) + self.offset)
             else:
-                self.coords[:, 0] -= (np.min(self.coords[:, 0]) + self.offset)
-        self.nn = _compute_normals(self.coords, self.faces)
+                coords[:, 0] -= (np.min(coords[:, 0]) + self.offset)
+        nn = _compute_normals(coords, faces)
 
-    def save_geometry(self):
-        surf_path = op.join(self.data_path, "surf",
-                            "%s.%s" % (self.hemi, self.surf))
-        nib.freesurfer.write_geometry(surf_path, self.coords, self.faces)
+        if self.coords is None:
+            self.coords = coords
+            self.faces = faces
+            self.nn = nn
+        else:
+            self.coords[:] = coords
+            self.faces[:] = faces
+            self.nn[:] = nn
 
     @property
     def x(self):
@@ -294,6 +332,28 @@ def set_log_file(fname=None, output_format='%(message)s', overwrite=None):
     logger.addHandler(lh)
 
 
+if hasattr(inspect, 'signature'):  # py35
+    def _get_args(function, varargs=False):
+        params = inspect.signature(function).parameters
+        args = [key for key, param in params.items()
+                if param.kind not in (param.VAR_POSITIONAL, param.VAR_KEYWORD)]
+        if varargs:
+            varargs = [param.name for param in params.values()
+                       if param.kind == param.VAR_POSITIONAL]
+            if len(varargs) == 0:
+                varargs = None
+            return args, varargs
+        else:
+            return args
+else:
+    def _get_args(function, varargs=False):
+        out = inspect.getargspec(function)  # args, varargs, keywords, defaults
+        if varargs:
+            return out[:2]
+        else:
+            return out[0]
+
+
 def verbose(function):
     """Decorator to allow functions to override default log level
 
@@ -310,7 +370,7 @@ def verbose(function):
         None defaults to using the current log level [e.g., set using
         mne.set_log_level()].
     """
-    arg_names = inspect.getargspec(function).args
+    arg_names = _get_args(function)
     # this wrap allows decorated functions to be pickled (e.g., for parallel)
 
     @wraps(function)
@@ -326,7 +386,7 @@ def verbose(function):
             # set it back if we get an exception
             try:
                 ret = function(*args, **kwargs)
-            except:
+            except Exception:
                 set_log_level(old_level)
                 raise
             set_log_level(old_level)
@@ -343,6 +403,12 @@ def verbose(function):
 ###############################################################################
 # USEFUL FUNCTIONS
 
+def _check_units(units):
+    if units not in ('m', 'mm'):
+        raise ValueError('Units must be "m" or "mm", got %r' % (units,))
+    return units
+
+
 def find_closest_vertices(surface_coords, point_coords):
     """Return the vertices on a surface mesh closest to some given coordinates.
 
@@ -365,25 +431,29 @@ def find_closest_vertices(surface_coords, point_coords):
     return np.argmin(cdist(surface_coords, point_coords), axis=0)
 
 
-def tal_to_mni(coords):
+def tal_to_mni(coords, units='mm'):
     """Convert Talairach coords to MNI using the Lancaster transform.
 
     Parameters
     ----------
     coords : n x 3 numpy array
         Array of Talairach coordinates
+    units : str
+        Can be 'm' or 'mm' (default).
 
     Returns
     -------
     mni_coords : n x 3 numpy array
-        Array of coordinates converted to MNI space
-
+        Array of coordinates converted to MNI space.
     """
     coords = np.atleast_2d(coords)
     xfm = np.array([[1.06860, -0.00396, 0.00826,  1.07816],
                     [0.00640,  1.05741, 0.08566,  1.16824],
                     [-0.01281, -0.08863, 1.10792, -4.17805],
                     [0.00000,  0.00000, 0.00000,  1.00000]])
+    units = _check_units(units)
+    if units == 'm':
+        xfm[:3, 3] /= 1000.
     mni_coords = np.dot(np.c_[coords, np.ones(coords.shape[0])], xfm.T)[:, :3]
     return mni_coords
 
@@ -415,7 +485,7 @@ def mesh_edges(faces):
     return edges
 
 
-def create_color_lut(cmap, n_colors=256):
+def create_color_lut(cmap, n_colors=256, center=None):
     """Return a colormap suitable for setting as a Mayavi LUT.
 
     Parameters
@@ -424,9 +494,16 @@ def create_color_lut(cmap, n_colors=256):
         Input colormap definition. This can be the name of a matplotlib
         colormap, a list of valid matplotlib colors, or a suitable
         mayavi LUT (possibly missing the alpha channel).
+
+        if value is "auto", a default sequential or divergent colormap is
+        returned
     n_colors : int, optional
         Number of colors in the resulting LUT. This is ignored if cmap
         is a 2d array.
+    center : double, optional
+        indicates whether desired colormap should be for divergent values,
+        currently only used to select default colormap for cmap='auto'
+
     Returns
     -------
     lut : n_colors x 4 integer array
@@ -444,24 +521,31 @@ def create_color_lut(cmap, n_colors=256):
 
             return lut
 
-    # Otherwise, we're going to try and use matplotlib to create it
+    # choose default colormaps (REMEMBER to change doc, e.g., in
+    # Brain.add_data, when changing these defaults)
+    if isinstance(cmap, string_types) and cmap == "auto":
+        if center is None:
+            cmap = "rocket"
+        else:
+            cmap = "icefire"
 
-    if cmap in dir(cm):
-        # This is probably a matplotlib colormap, so build from that
-        # The matplotlib colormaps are a superset of the mayavi colormaps
-        # except for one or two cases (i.e. blue-red, which is a crappy
-        # rainbow colormap and shouldn't be used for anything, although in
-        # its defense it's better than "Jet")
-        cmap = getattr(cm, cmap)
-
-    elif np.iterable(cmap):
-        # This looks like a list of colors? Let's try that.
-        colors = list(map(mpl.colors.colorConverter.to_rgb, cmap))
-        cmap = mpl.colors.LinearSegmentedColormap.from_list("_", colors)
+    surfer_cmaps = ["rocket", "mako", "icefire", "vlag"]
+    surfer_cmaps += [name + "_r" for name in surfer_cmaps]
 
+    if not isinstance(cmap, string_types) and isinstance(cmap, Sequence):
+        colors = list(map(mpl.colors.colorConverter.to_rgba, cmap))
+        cmap = mpl.colors.ListedColormap(colors)
+    elif cmap in surfer_cmaps:
+        cmap = getattr(surfer_cm, cmap)
     else:
-        # If we get here, it's a bad input
-        raise ValueError("Input %s was not valid for making a lut" % cmap)
+        try:
+            # Try to get a named matplotlib colormap
+            # This will also pass Colormap object back out
+            cmap = mpl_cm.get_cmap(cmap)
+        except (TypeError, ValueError):
+            # If we get here, it's a bad input
+            # but don't raise the matplotlib error as it is less accurate
+            raise ValueError("Input %r was not valid for making a lut" % cmap)
 
     # Convert from a matplotlib colormap to a lut array
     lut = (cmap(np.linspace(0, 1, n_colors)) * 255).astype(np.int)
@@ -534,7 +618,8 @@ def smoothing_matrix(vertices, adj_mat, smoothing_steps=20, verbose=None):
 
 @verbose
 def coord_to_label(subject_id, coord, label, hemi='lh', n_steps=30,
-                   map_surface='white', coord_as_vert=False, verbose=None):
+                   map_surface='white', coord_as_vert=False, units='mm',
+                   verbose=None):
     """Create label from MNI coordinate
 
     Parameters
@@ -553,18 +638,24 @@ def coord_to_label(subject_id, coord, label, hemi='lh', n_steps=30,
         The surface name used to find the closest point
     coord_as_vert : bool
         whether the coords parameter should be interpreted as vertex ids
+    units : str
+        Can be 'm' or 'mm' (default).
     verbose : bool, str, int, or None
         If not None, override default verbose level (see surfer.verbose).
     """
-    geo = Surface(subject_id, hemi, map_surface)
+    geo = Surface(subject_id, hemi, map_surface, units=units)
     geo.load_geometry()
 
+    coords = geo.coords
+    # work in mm from here on
+    if geo.units == 'm':
+        coords = coords * 1000
     if coord_as_vert:
-        coord = geo.coords[coord]
+        coord = coords[coord]
 
-    n_vertices = len(geo.coords)
+    n_vertices = len(coords)
     adj_mat = mesh_edges(geo.faces)
-    foci_vtxs = find_closest_vertices(geo.coords, [coord])
+    foci_vtxs = find_closest_vertices(coords, [coord])
     data = np.zeros(n_vertices)
     data[foci_vtxs] = 1.
     smooth_mat = smoothing_matrix(np.arange(n_vertices), adj_mat, 1)
@@ -578,7 +669,7 @@ def coord_to_label(subject_id, coord, label, hemi='lh', n_steps=30,
     f.write('#label at %s from subject %s\n' % (coord, subject_id))
     f.write('%d\n' % len(idx))
     for i in idx:
-        x, y, z = geo.coords[i]
+        x, y, z = coords[i]
         f.write('%d  %f  %f  %f 0.000000\n' % (i, x, y, z))
 
 
@@ -614,27 +705,43 @@ def _get_subjects_dir(subjects_dir=None, raise_error=True):
     return subjects_dir
 
 
-def has_fsaverage(subjects_dir=None):
+def has_fsaverage(subjects_dir=None, raise_error=True, return_why=False):
     """Determine whether the user has a usable fsaverage"""
-    fs_dir = op.join(_get_subjects_dir(subjects_dir, False), 'fsaverage')
-    if not op.isdir(fs_dir):
-        return False
-    if not op.isdir(op.join(fs_dir, 'surf')):
-        return False
-    return True
-
-
-def has_imageio():
-    try:
-        import imageio  # NOQA
-    except ImportError:
-        return False
+    subjects_dir = _get_subjects_dir(subjects_dir, raise_error=raise_error)
+    out = ''
+    if not op.isdir(subjects_dir):
+        out = 'SUBJECTS_DIR not found: %s' % (subjects_dir,)
     else:
-        return True
+        fs_dir = op.join(_get_subjects_dir(subjects_dir, False), 'fsaverage')
+        surf_dir = op.join(fs_dir, 'surf')
+        if not op.isdir(fs_dir):
+            out = 'fsaverage not found in SUBJECTS_DIR: %s' % (fs_dir,)
+        elif not op.isdir(surf_dir):
+            out = 'fsaverage has no "surf" directory: %s' % (surf_dir,)
+    out = (out == '', out) if return_why else (out == '')
+    return out
 
 
-requires_fsaverage = np.testing.dec.skipif(not has_fsaverage(),
-                                           'Requires fsaverage subject data')
+def requires_fsaverage():
+    import pytest
+    has, why = has_fsaverage(raise_error=False, return_why=True)
+    return pytest.mark.skipif(
+        not has, reason='Requires fsaverage subject data (%s)' % why)
 
-requires_imageio = np.testing.dec.skipif(not has_imageio(),
-                                         "Requires imageio package")
+
+def requires_imageio():
+    import pytest
+    try:
+        from imageio.plugins.ffmpeg import get_exe  # noqa, analysis:ignore
+    except ImportError:
+        has = False
+    else:
+        has = True
+    return pytest.mark.skipif(not has, reason="Requires imageio with ffmpeg")
+
+
+def requires_fs():
+    import pytest
+    has = ('FREESURFER_HOME' in os.environ)
+    return pytest.mark.skipif(
+        not has, reason='Requires FreeSurfer command line tools')
